/***************************************************************************
 *   Copyright (C) 2005 by yunfan                                          *
 *   yunfan_zg@163.com                                                     *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "evachatview.h"
#include "evahtmlparser.h"
#include "evaresource.h"
#include "../evamain.h"
#include <qstringlist.h>
#include <qtimer.h>
#include <qpopupmenu.h>
#include <qclipboard.h>
#include <qdragobject.h>
#include <kaction.h>
#include <qapplication.h>
#include <kapplication.h>
#include <klocale.h>

const QString EvaChatView::protocolAccept = "cmd://accept";
const QString EvaChatView::protocolSaveAs = "cmd://saveas";
const QString EvaChatView::protocolCancel = "cmd://cancel";
const QString EvaChatView::protocolResume = "cmd://resume";
const QString EvaChatView::protocolNewOne = "cmd://newone";

EvaChatView::EvaChatView( QWidget * parent, const char * name )
	: KHTMLPart(parent, name), menu(NULL), buffer(""), 
	lastHeight(0)
{
	//setOnlyLocalReferences(true);
	//if(menu) delete menu;
	menu = new QPopupMenu(0, "popup");
	copyAction  = KStdAction::copy( this, SLOT(copy()), actionCollection());
	copyAction->plug( menu );
	setDNDEnabled(true);
	setAutoloadImages(true);
	view()->setHScrollBarMode(QScrollView::AlwaysOff);
	
	//setStandardFont("Helvetica");
	//buffer += "<body style=\"font-size:9pt;font-family:Helvetica\">";
	QObject::connect(this, SIGNAL(selectionChanged()), SLOT(slotSelectionChanged()));
	QObject::connect(this, SIGNAL(popupMenu(const QString &, const QPoint &)), 
			SLOT(slotPopupMenu(const QString &, const QPoint &)));
	QObject::connect(browserExtension(), SIGNAL(openURLRequest(const KURL &, const KParts::URLArgs &)), 
			SLOT(slotLinkClicked( const KURL &)));
}

EvaChatView::~EvaChatView()
{
	delete menu;
}

QString EvaChatView::wrapFontAttributes(QColor color, Q_UINT8 size,
                                            bool underline, bool italic, bool bold,
                                            QString contents)
{
	QString fontHead = "<span style=\"";
	QString fontSize = "font-size: "+QString::number(size)+"pt; ";

	QString fontColor = QString("color: ") + color.name() + QString("\" >");
	
	QString fontStye = "", fontStyeEnd="";
	if(bold){
		fontStye+="<b>";
		fontStyeEnd+="</b>";
	}
	if(italic){
		fontStye+="<i>";
		fontStyeEnd="</i>" + fontStyeEnd;
	}
	if(underline){
		fontStye+="<u>";
		fontStyeEnd="</u>" + fontStyeEnd;
	}
	QString fontEnd = "</span>";
	QString ret = fontHead + fontSize + fontColor + fontStye + contents + fontStyeEnd + fontEnd;
	return ret;
}

QString EvaChatView::wrapNickName(QString &nick, QDateTime time, QColor color, bool isNormal)
{	
	QString htmlName = nick;
	EvaHtmlParser parser;
	parser.setAbsImagePath(EvaMain::images->getSmileyPath());
	parser.convertToHtml(htmlName, false, true);
	QString msg = "<span style=\"font-size: 9pt; color: " + color.name() +"\">" + htmlName + " ";
	if(!isNormal)
		msg+=i18n("(Auto-Reply)");
	
	msg+=" "+time.toString("yyyy-MM-dd hh:mm:ss") + "</span><br>";
	return msg;
}

void EvaChatView::append( QString & nick, QDateTime time, QColor nameColor, bool isNormal, 
				QColor msgColor, Q_UINT8 size, 
				bool underline, bool italic, bool bold, QString contents )
{
	QString msg = wrapNickName(nick, time, nameColor, isNormal) + 
			wrapFontAttributes(msgColor, size, underline, italic, bold, contents);
// 	buffer += (msg + "<br>");
// 	begin();
// 	write(buffer);
// 	write("</body>");
// 	end();
// 	
// 	lastHeight = view()->contentsHeight();
// 	QTimer::singleShot(20, this, SLOT(slotScrollToBottom()));
	updateContents(msg);
}

void EvaChatView::updatePicture( const QString filename , const QString tmpFileName)
{
// 	printf("EvaChatView::updatePicture -- filename:%s\ntmpFileName:%s\n", filename.ascii(), tmpFileName.ascii());
// 	printf("EvaChatView::updatePicture -- before\n%s\n", buffer.ascii());
	buffer.replace(tmpFileName, filename);
// 	printf("EvaChatView::updatePicture -- after\n%s\n", buffer.ascii());
	begin();
	write(buffer);
	write("</body>");
	end();
	
	lastHeight = view()->contentsHeight();
	QTimer::singleShot(20, this, SLOT(slotScrollToBottom()));
}

void EvaChatView::slotScrollToBottom()
{
	if( view()->contentsHeight() == lastHeight){	
		QTimer::singleShot(20, this, SLOT(slotScrollToBottom()));
	}else{
		view()->setContentsPos(view()->contentsX(), view()->contentsHeight() - view()->visibleHeight() );
		lastHeight = view()->contentsHeight();
	}
}

void EvaChatView::slotLinkClicked( const KURL & url)
{
	QString cmd = url.url();
//	QString name = cmd.right(cmd.length() - cmd.find("_") -1);
	QString strSession = cmd.mid(13,cmd.length() - 13);
	bool ok;
	unsigned int session = strSession.toUInt(&ok);
	if(ok){ 
		//printf("EvaChatView::slotLinkClicked --- name:%s\n", strSession.ascii());
		if(cmd.startsWith(protocolAccept) ){
			emit fileTransferAcceptRequest(session);
			return;
		}
		if(cmd.startsWith(protocolSaveAs) ){
			emit fileTransferSaveAsRequest(session);
			return;
		}
		if(cmd.startsWith(protocolCancel) ){
			emit fileTransferCancelRequest(session);
			return;
		}
		if(cmd.startsWith(protocolResume) ){
			emit fileTransferResume(session, true);
			return;
		}
		if(cmd.startsWith(protocolNewOne) ){
			emit fileTransferResume(session, false);
			return;
		}
	}

	QStringList args;
	args<<"exec"<< cmd;printf("cmd: %s\n", cmd.ascii());
	kapp->kdeinitExec("kfmclient",args);
}

void EvaChatView::slotPopupMenu( const QString & , const QPoint & point )
{
	//copyAction->setEnabled( hasSelection() );
	menu->popup(point);
}

void EvaChatView::copy( )
{
	if(hasSelection()){
		QString text = selectedText();
		QApplication::clipboard()->setText( text, QClipboard::Clipboard );
		QApplication::clipboard()->setText( text, QClipboard::Selection );
	}
}

// it seems this method doesn't do the job
void EvaChatView::startDrag( )
{
	QDragObject *d = new QTextDrag(selectedText(), view());
	d->dragCopy();
}

void EvaChatView::slotSelectionChanged( )
{
	copyAction->setEnabled( hasSelection() );
}

void EvaChatView::showInfomation( const QString & info )
{
	QString picPath = "<img src = \"" + EvaMain::images->getIconFullPath("MSG_INFO") + "\">";
	QString msg = wrapFontAttributes(Qt::gray, 9, false, false, false, info);
	updateContents("&nbsp;" + picPath + "&nbsp;" + msg );
}

void EvaChatView::showFileNotification( const QString & who, const QString & filename, 
			const int size, const unsigned int session, const bool isSend )
{
	QString strSession = QString::number(session);
	QString picPath = "<img src = \"" + EvaMain::images->getIconFullPath("MSG_INFO") + "\">";
	QString acceptLink = "<a href=\""+ protocolAccept + "_" + strSession + "\">" + i18n("Accept") + "</a>";
	QString saveAsLink = "<a href=\""+ protocolSaveAs + "_" + strSession + "\">" + i18n("Save As") + "</a>";
        QString rejectLink = "<a href=\""+ protocolCancel + "_" + strSession + "\">" + i18n("Reject") + "</a>";
	QString cancelLink = "<a href=\""+ protocolCancel + "_" + strSession + "\">" + i18n("Cancel") + "</a>";
	QString fileSize = (size<0x400)?(QString::number(size) + "B"):
			((size<0x100000)?(QString::number(size/0x400) + "KB") :
			((size<0x40000000)?(QString::number(size/0x100000) + "MB"):
			(QString::number(size/0x40000000) + "GB") ) );
	QString fileInfo = filename + "(" + fileSize + ")";
        QString txt;
	if(isSend){
		txt = i18n("Waiting for %1 accepting file \"%2\". Please wait or %3.")
				.arg(who).arg(fileInfo).arg(cancelLink);
	}else{
		txt = i18n("%1 wants to send you a file \"%2\", you can %3, %4 or %5.")
				.arg(who).arg(fileInfo).arg(acceptLink).arg(saveAsLink).arg(rejectLink);
	}
	QString msg = wrapFontAttributes(Qt::gray, 9, false, false, false, txt);
	updateContents("&nbsp;" + picPath + "&nbsp;" + msg );
}

void EvaChatView::updateContents( const QString & contents )
{
	//printf("EvaChatView::updateContents -- \n%s\n", contents.ascii());
	QString newMsg = contents;
	newMsg.replace("&nbsp;", "&nbsp; ");
	buffer += (newMsg + "<br>");
	begin();
	write(buffer);
	write("</body>");
	end();
	
	lastHeight = view()->contentsHeight();
	QTimer::singleShot(20, this, SLOT(slotScrollToBottom()));
}

void EvaChatView::askResumeMode( const QString filename, const unsigned int session )
{
	QString strSession = QString::number(session);
	QString picPath = "<img src = \"" + EvaMain::images->getIconFullPath("MSG_INFO") + "\">";
	QString resumeLink = "<a href=\""+ protocolResume + "_" + strSession + "\">" + 
			i18n("resume") + "</a>";
	QString restartLink = "<a href=\""+ protocolNewOne + "_" + strSession + "\">" + 
			i18n("start") + "</a>";
	QString txt = i18n("Cached information of \"%1\" has been found, you can %2 the last download or ignore the last cached download information and %3 a new download.").arg(filename).arg(resumeLink).arg(restartLink);

	QString msg = wrapFontAttributes(Qt::gray, 9, false, false, false, txt);
	updateContents("&nbsp;" + picPath + "&nbsp;" + msg );
}



